import pytest


@pytest.mark.skip(reason="bad API_KEY on old aus east")
def test_get_sample_ecommerce_dataset():
    from relevanceai.datasets import get_ecommerce_1_dataset

    assert len(get_ecommerce_1_dataset(number_of_documents=100)) == 100



"""
Simple tests to ensure import errors don't happen
"""


def test_client_smoke():
    """Simple test to ensure there are no breaking installations."""
    # Import the client
    from relevanceai import Client

    assert True


def test_datasets_smoke():
    from relevanceai.datasets import get_games_dataset
    from relevanceai.datasets import get_ecommerce_1_dataset
    from relevanceai.datasets import get_online_retail_dataset
    from relevanceai.datasets import get_news_dataset
    from relevanceai.datasets import get_ecommerce_3_dataset

    assert True


def test_projector_smoke():
    import relevanceai.vector_tools.constants
    from relevanceai.vector_tools.dim_reduction import DimReduction
    from relevanceai.vector_tools.cluster import Cluster
    from relevanceai.visualise.projector import Projector

    assert True



import pytest

from typing import Dict, List, NamedTuple

from relevanceai import Client

from tests.globals.constants import generate_dataset_id


@pytest.fixture(scope="function")
def obj_dataset_id(test_client: Client, dataclass_documents: List[NamedTuple]):
    test_dataset_id = generate_dataset_id()

    test_client._insert_documents(test_dataset_id, dataclass_documents)

    yield test_dataset_id

    test_client.datasets.delete(test_dataset_id)


@pytest.fixture(scope="function")
def clustered_dataset_id(test_client: Client, vector_documents: List[Dict]):
    test_dataset_id = generate_dataset_id()

    test_client._insert_documents(test_dataset_id, vector_documents)

    test_client.vector_tools.cluster.kmeans_cluster(
        dataset_id=test_dataset_id,
        vector_fields=["sample_1_vector_"],
        k=10,
        alias="kmeans_10",
        overwrite=True,
    )

    yield test_dataset_id

    test_client.datasets.delete(test_dataset_id)



from typing import List, Dict

from doc_utils import DocUtils
from relevanceai.base import _Base
from relevanceai.api.endpoints.client import APIClient


class Utils(APIClient, _Base, DocUtils):
    def __init__(self, project: str, api_key: str, firebase_uid: str):
        self.project = project
        self.api_key = api_key
        self.firebase_uid = firebase_uid

        super().__init__(project=project, api_key=api_key, firebase_uid=firebase_uid)

    def _is_valid_vector_name(self, dataset_id, vector_name: str) -> bool:
        """
        Check vector field name is valid
        """
        vector_fields = self.get_vector_fields(dataset_id)
        schema = self.datasets.schema(dataset_id)
        if vector_name in schema.keys():
            if vector_name in vector_fields:
                return True
            else:
                raise ValueError(f"{vector_name} is not a valid vector name")
        else:
            raise ValueError(f"{vector_name} is not in the {dataset_id} schema")

    def _is_valid_label_name(self, dataset_id, label_name: str) -> bool:
        """
        Check vector label name is valid. Checks that it is either numeric or text
        """
        schema = self.datasets.schema(dataset_id)
        if label_name == "_id":
            return True
        if label_name in list(schema.keys()):
            if schema[label_name] in ["numeric", "text"]:
                return True
            else:
                raise ValueError(f"{label_name} is not a valid label name")
        else:
            raise ValueError(f"{label_name} is not in the {dataset_id} schema")

    def _remove_empty_vector_fields(self, documents, vector_field: str) -> List[Dict]:
        """
        Remove documents with empty vector fields
        """
        return [d for d in documents if d.get(vector_field)]

    def _convert_id_to_string(self, documents):
        self.set_field_across_documents(
            "_id", [str(i["_id"]) for i in documents], documents
        )

    def _are_fields_in_schema(self, fields, dataset_id, schema=None):
        """
        Check fields are in schema
        """
        if schema is None:
            schema = self.datasets.schema(dataset_id)
        invalid_fields = []
        for i in fields:
            if i not in schema:
                invalid_fields.append(i)
        if len(invalid_fields) > 0:
            raise ValueError(
                f"{', '.join(invalid_fields)} are invalid fields. They are not in the dataset schema."
            )
        return



"""
Automated Cluster Reporting

.. warning::
    This is a beta feature.

.. note::
    **Introduced in v1.0.0.**


.. code-block::

    import requests
    docs = requests.get("https://raw.githubusercontent.com/fanzeyi/pokemon.json/master/pokedex.json").json()
    for d in docs:
        b = d['base']
        d.update(b)
        d['base_vector_'] = [b["Attack"], b["Defense"], b["HP"], b["Sp. Attack"], b["Sp. Defense"], b["Speed"]]

    import pandas as pd
    import numpy as np
    df = pd.DataFrame(docs)
    X = np.array(df['base_vector_'].tolist())


    from relevanceai.cluster_report.cluster_report import ClusterReport
    from sklearn.cluster import KMeans

    N_CLUSTERS = 2
    kmeans = KMeans(n_clusters=N_CLUSTERS)
    cluster_labels = kmeans.fit_predict(X)

    report = ClusterReport(
        X=X, 
        cluster_labels=cluster_labels, 
        num_clusters=N_CLUSTERS, 
        model=kmeans
    )

    internal_report = report.get_cluster_internal_report()

"""

import pandas as pd
import numpy as np
from relevanceai.warnings import warn_function_is_work_in_progress
from typing import Union, List, Dict, Any

try:
    from sklearn.metrics import (
        davies_bouldin_score,
        calinski_harabasz_score,
        silhouette_samples,
    )
    from sklearn.metrics.pairwise import (
        pairwise_distances,
    )
    from sklearn.tree import _tree, DecisionTreeClassifier
    from sklearn.cluster import KMeans
except ModuleNotFoundError as e:
    pass


class ClusterReport:
    """
    Receive an automated cluster reprot

    .. warning::
        This is a beta feature.

    .. note::
        **Introduced in v1.0.0.**


    Parameters
    -------------

    X: np.ndarray
        The original data
    cluster_labels: List[str]
        A list of cluster labels
    model
        The model to analyze. Currently only used
    num_clusters: Optional[int]
        The number of clusters. This is required if we can't actually tell how many clusters there are

    """

    def __init__(
        self,
        X: Union[list, np.ndarray],
        cluster_labels: List[Union[str, float]],
        model: KMeans = None,
        num_clusters: int = None,
    ):
        warn_function_is_work_in_progress()
        if isinstance(X, list):
            self.X = np.array(X)
        else:
            self.X = X
        if isinstance(cluster_labels, list):
            self.cluster_labels = np.array(cluster_labels)
        else:
            self.cluster_labels = cluster_labels
        self.num_clusters = (
            len(cluster_labels) if num_clusters is None else num_clusters
        )
        self.model = model

    @staticmethod
    def summary_statistics(array: np.ndarray, axis=0):
        """
        Basic summary statistics
        """
        if axis == 2:
            return {
                "sum": array.sum(),
                "mean": array.mean(),
                "std": array.std(),
                "variance": array.var(),
                "min": array.min(),
                "max": array.max(),
                "12_5%": np.percentile(array, 12.5),
                "25%": np.percentile(array, 25),
                "37_5%": np.percentile(array, 37.5),
                "50%": np.percentile(array, 50),
                "62_5%": np.percentile(array, 62.5),
                "75%": np.percentile(array, 75),
                "87_5%": np.percentile(array, 87.5),
            }
        else:
            return {
                "sum": array.sum(axis=axis),
                "mean": array.mean(axis=axis),
                "std": array.std(axis=axis),
                "variance": array.var(axis=axis),
                "min": array.min(axis=axis),
                "max": array.max(axis=axis),
                "12_5%": np.percentile(array, 12.5, axis=axis),
                "25%": np.percentile(array, 25, axis=axis),
                "37_5%": np.percentile(array, 37.5, axis=axis),
                "50%": np.percentile(array, 50, axis=axis),
                "62_5%": np.percentile(array, 62.5, axis=axis),
                "75%": np.percentile(array, 75, axis=axis),
                "87_5%": np.percentile(array, 87.5, axis=axis),
            }

    def get_distance_from_centroid(self, cluster_data, center_vector):
        distances_from_centroid = pairwise_distances([center_vector], cluster_data)
        return ClusterReport.summary_statistics(distances_from_centroid, axis=2)

    def get_distance_from_centroid_to_another(self, other_cluster_data, center_vector):
        """Store the distances from a centroid to another."""
        distances_from_centroid_to_another = pairwise_distances(
            [center_vector], other_cluster_data
        )
        return ClusterReport.summary_statistics(
            distances_from_centroid_to_another, axis=2
        )

    def get_distance_from_grand_centroid(self, grand_centroid, specific_cluster_data):
        distances_from_grand_centroid = pairwise_distances(
            [grand_centroid], specific_cluster_data
        )
        return ClusterReport.summary_statistics(distances_from_grand_centroid, axis=2)

    def get_distance_from_grand_centroid_to_point_in_another_cluster(
        self, grand_centroid, other_cluster_data
    ):
        distances_from_grand_centroid_to_another = pairwise_distances(
            [grand_centroid], other_cluster_data
        )
        return ClusterReport.summary_statistics(
            distances_from_grand_centroid_to_another, axis=2
        )

    @staticmethod
    def get_z_score(value, mean, std):
        return (value - mean) / std

    # TODO: Implement caching
    def get_centers(self):
        # Here we add support for both RelevanceAI cluster models
        # but also regular sklearn cluster models
        if hasattr(self.model, "cluster_centers_"):
            return self.model.cluster_centers_
        elif hasattr(self.model, "get_centers"):
            return self.model.get_centers()
        else:
            import warnings

            warnings.warn("Missing centroids.")
            return

    def get_cluster_internal_report(self):
        """
        Provide the standard clustering report.
        """
        self.X_silhouette_scores = silhouette_samples(
            self.X, self.cluster_labels, metric="euclidean"
        )
        self.cluster_internal_report = {
            "overall": {
                "summary": ClusterReport.summary_statistics(self.X),
                "davies_bouldin_score": davies_bouldin_score(
                    self.X, self.cluster_labels
                ),
                "calinski_harabasz_score": calinski_harabasz_score(
                    self.X, self.cluster_labels
                ),
                "silhouette_score": ClusterReport.summary_statistics(
                    self.X_silhouette_scores
                ),
            },
            "each": {
                "summary": {},
                "centers": {},
                "silhouette_score": {},
            },
        }
        self._store_basic_centroid_stats(self.cluster_internal_report["overall"])

        label_value_counts = np.unique(self.cluster_labels, return_counts=True)

        cluster_report = {"frequency": {"total": 0, "each": {}}}

        for i, cluster_label in enumerate(label_value_counts[0]):
            cluster_bool = self.cluster_labels == cluster_label

            specific_cluster_data = self.X[cluster_bool]
            other_cluster_data = self.X[~cluster_bool]

            cluster_frequency = label_value_counts[1][i]
            cluster_report["frequency"]["total"] += cluster_frequency
            cluster_report["frequency"]["each"][cluster_label] = cluster_frequency

            self.cluster_internal_report["each"]["summary"][
                cluster_label
            ] = ClusterReport.summary_statistics(self.X)

            # If each value of the vector is important
            center_stats = {"by_features": {}}

            center_stats["by_features"]["summary"] = ClusterReport.summary_statistics(
                specific_cluster_data
            )

            squared_errors = np.square(
                np.subtract(
                    [self.get_centers()[i]] * len(self.X[cluster_bool]),
                    self.X[cluster_bool],
                )
            )
            if self.has_centers():

                grand_centroid = self.X[cluster_bool].mean(axis=0)
                self.cluster_internal_report["overall"]["grand_centroids"].append(
                    grand_centroid
                )

                center_stats[
                    "distance_from_centroid"
                ] = self.get_distance_from_centroid(
                    specific_cluster_data, self.get_centers()[i]
                )

                center_stats[
                    "distance_from_centroid_to_point_in_another_cluster"
                ] = self.get_distance_from_centroid_to_another(
                    other_cluster_data, self.get_centers()[i]
                )

                center_stats["distances_from_grand_centroid"] = pairwise_distances(
                    [grand_centroid], specific_cluster_data
                )

                center_stats[
                    "distance_from_grand_centroid"
                ] = self.get_distance_from_grand_centroid(
                    grand_centroid, specific_cluster_data
                )

                center_stats[
                    "distance_from_grand_centroid_to_point_in_another_cluster"
                ] = self.get_distance_from_grand_centroid_to_point_in_another_cluster(
                    grand_centroid, other_cluster_data
                )

                center_stats["by_features"][
                    "overall_z_score"
                ] = ClusterReport.get_z_score(
                    self.get_centers()[i],
                    self.cluster_internal_report["overall"]["summary"]["mean"],
                    self.cluster_internal_report["overall"]["summary"]["std"],
                )

                center_stats["by_features"]["z_score"] = ClusterReport.get_z_score(
                    self.get_centers()[i],
                    self.cluster_internal_report["each"]["summary"][cluster_label][
                        "mean"
                    ],
                    self.cluster_internal_report["each"]["summary"][cluster_label][
                        "std"
                    ],
                )

                center_stats["by_features"][
                    "overall_z_score_grand_centroid"
                ] = ClusterReport.get_z_score(
                    grand_centroid,
                    self.cluster_internal_report["overall"]["summary"]["mean"],
                    self.cluster_internal_report["overall"]["summary"]["std"],
                )

                center_stats[
                    "overall_z_score_grand_centroid"
                ] = ClusterReport.summary_statistics(
                    center_stats["by_features"]["overall_z_score_grand_centroid"]
                )

                center_stats["by_features"]["z_score_grand_centroid"] = (
                    grand_centroid
                    - self.cluster_internal_report["each"]["summary"][cluster_label][
                        "mean"
                    ]
                ) / self.cluster_internal_report["each"]["summary"][cluster_label][
                    "std"
                ]

                # this might not be needed
                center_stats["overall_z_score"] = ClusterReport.summary_statistics(
                    center_stats["by_features"]["overall_z_score"]
                )

                # this might not be needed
                center_stats["z_score"] = ClusterReport.summary_statistics(
                    center_stats["by_features"]["z_score"]
                )
                center_stats[
                    "z_score_grand_centroid"
                ] = ClusterReport.summary_statistics(
                    center_stats["by_features"]["z_score_grand_centroid"]
                )

                center_stats["squared_errors"] = ClusterReport.summary_statistics(
                    squared_errors, axis=2
                )

            squared_errors_by_col = {}

            for f in range(len(squared_errors[0])):
                squared_errors_by_col[f] = ClusterReport.summary_statistics(
                    squared_errors[:, f], axis=2
                )

            center_stats["by_features"]["squared_errors"] = squared_errors_by_col
            self.cluster_internal_report["each"]["centers"][
                cluster_label
            ] = center_stats
            self.cluster_internal_report["each"]["silhouette_score"][
                cluster_label
            ] = ClusterReport.summary_statistics(
                self.X_silhouette_scores[cluster_bool], axis=2
            )

        if self.has_centers():
            self.cluster_internal_report["overall"]["dunn_index"] = (
                min(
                    c["distance_from_centroid"]["min"]
                    for c in self.cluster_internal_report["each"]["centers"].values()
                )
                / self.cluster_internal_report["overall"][
                    "centroids_distance_matrix"
                ].max()
            )

        return self.cluster_internal_report

    def has_centers(self):
        return self.get_centers() is not None

    def _store_basic_centroid_stats(self, overall_report):
        """Store"""
        if self.has_centers():
            overall_report["centroids"] = self.get_centers()
            overall_report["centroids_distance_matrix"] = pairwise_distances(
                self.get_centers(), metric="euclidean"
            )
            overall_report["grand_centroids"] = []
            overall_report["average_distance_between_centroids"] = (
                overall_report["centroids_distance_matrix"].sum(axis=1) - 1
            ) / self.num_clusters

    def get_class_rules(self, tree: DecisionTreeClassifier, feature_names: list):
        self.inner_tree: _tree.Tree = tree.tree_
        self.classes = tree.classes_
        self.class_rules_dict: Dict[Any, Any] = dict()
        self.tree_dfs()

    def tree_dfs(self, node_id=0, current_rule=[]):
        if not hasattr(self, "classes"):
            self.get_class_rules()

        # feature[i] holds the feature to split on, for the internal node i.
        split_feature = self.inner_tree.feature[node_id]
        if split_feature != _tree.TREE_UNDEFINED:  # internal node
            name = self.feature_names[split_feature]
            threshold = self.inner_tree.threshold[node_id]
            # left child
            left_rule = current_rule + ["({} <= {})".format(name, threshold)]
            self.tree_dfs(self.inner_tree.children_left[node_id], left_rule)
            # right child
            right_rule = current_rule + ["({} > {})".format(name, threshold)]
            self.tree_dfs(self.inner_tree.children_right[node_id], right_rule)
        else:  # leaf
            dist = self.inner_tree.value[node_id][0]
            dist = dist / dist.sum()
            max_idx = dist.argmax()
            if len(current_rule) == 0:
                rule_string = "ALL"
            else:
                rule_string = " and ".join(current_rule)
            # register new rule to dictionary
            selected_class = self.classes[max_idx]
            class_probability = dist[max_idx]
            class_rules = self.class_rules_dict.get(selected_class, [])
            class_rules.append((rule_string, class_probability))
            self.class_rules_dict[selected_class] = class_rules

    def cluster_reporting(self, data: pd.DataFrame, clusters, max_depth: int = 5):
        # Create Model
        tree = DecisionTreeClassifier(max_depth=max_depth, criterion="entropy")
        tree.fit(data, clusters)
        print(tree.score(data, clusters))

        # Generate Report
        self.feature_names = data.columns
        self.get_class_rules(tree, self.feature_names)

        report_class_list = []

        for class_name in self.class_rules_dict.keys():
            rule_list = self.class_rules_dict[class_name]
            combined_string = ""
            for rule in rule_list:
                combined_string += "[{}] {}\n\n".format(rule[1], rule[0])
            report_class_list.append((class_name, combined_string))

        cluster_instance_df = pd.Series(clusters).value_counts().reset_index()
        cluster_instance_df.columns = ["class_name", "instance_count"]

        report_df = pd.DataFrame(report_class_list, columns=["class_name", "rule_list"])
        report_df = pd.merge(
            cluster_instance_df, report_df, on="class_name", how="left"
        )
        return report_df.sort_values(by="class_name")[
            ["class_name", "instance_count", "rule_list"]
        ]



"""
Migrate from mongo database to Relevance Ai:

.. code-block::

    # Create an object of MongoImporter class
    connection_string= "..."
    project= "..."
    api_key= "..."
    mongo_importer = MongoImporter(connection_string, project, api_key)

    # Get a summary of the mondo database using "mongo_summary"
    mongo_importer.mongo_summary()

    # Set the desired source mongo collection using "set_mongo_collection"
    db_name = '...'
    collection_name = '...'
    mongo_importer.set_mongo_collection(db_name, dataset_id)

    # Get total number of entries in the mongo collection using "mongo_document_count"
    document_count = mongo_importer.mongo_document_count()

    # Migrate data from mongo to Relevance AI using "migrate_mongo2relevance_ai"
    chunk_size = 5000      # migrate batches of 5000 (default 2000)
    start_idx= 12000       # loads from mongo starting at index 12000 (default 0)
    dataset_id = "..."     # dataset id in the Relevance Ai platform
    mongo_importer.migrate(
        dataset_id, document_count, chunk_size=chunk_size,
        start_idx=start_idx)

"""

import copy
import json
import math
import numpy as np
import pandas as pd
import uuid
import warnings
from tqdm.auto import tqdm
from typing import List
from relevanceai.api.client import BatchAPIClient

try:
    from pymongo import MongoClient

    PYMONGO_AVAILABLE = True
except (ImportError, ModuleNotFoundError):
    PYMONGO_AVAILABLE = False
    warnings.warn(
        "you are missing `pymongo.MongoClient`. Please install this using `pip install pymongo==3.12`"
    )

try:
    from bson import json_util

    BSON_AVAILABLE = True
except (ImportError, ModuleNotFoundError):
    BSON_AVAILABLE = False
    warnings.warn(
        "you are missing `bson.json_util`. Please install this using `pip install bson`"
    )


class MongoImporter(BatchAPIClient):
    def __init__(
        self, connection_string: str, project: str, api_key: str, firebase_uid: str
    ):
        super().__init__(project=project, api_key=api_key, firebase_uid=firebase_uid)
        if PYMONGO_AVAILABLE:
            self.mongo_client = MongoClient(connection_string)
        else:
            self.logger.error(
                "you are missing Pymongo. Please install this using `pip install pymongo==3.12`"
            )

    def mongo_summary(self):
        """
        returns a dictionary {key:value}
        key = db names
        value = collection names in each db
        """
        summary = {}
        for db in self.mongo_client.database_names():
            summary[db] = []
            for collection in self.mongo_client[db].collection_names():
                summary[db].append(collection)
        return summary

    def get_mongo_db(self, db_name: str):
        return self.mongo_client[db_name]

    def get_mongo_collection(self, db_name: str, collection_name: str):
        return self.mongo_client[db_name][collection_name]

    def set_mongo_db(self, db_name: str):
        self.mongodb = self.mongo_client[db_name]

    def set_mongo_collection(self, db_name: str, collection_name: str):
        self.mongo_collection = self.mongo_client[db_name][collection_name]

    def mongo_document_count(self):
        return self.mongo_collection.count()

    def create_relevance_ai_dataset(self, dataset_id: str):
        response = self.datasets.create(dataset_id)
        return response

    def update_id(self, documents: List[dict]):
        # makes bson id format json campatible
        for doc in documents:
            try:
                doc["_id"] = doc["_id"]["$oid"]
            except Exception as e:
                self.logger.info("Could not use the original id: " + str(e))
                doc["_id"] = uuid.uuid4().__str__()
        return documents

    @staticmethod
    def parse_json(data):
        return json.loads(json_util.dumps(data))

    @staticmethod
    def flatten_inner_indxs(documents: List[dict]):
        # {f1:{f2:v}} => {f1-f2:v}
        expanded = copy.deepcopy(documents)
        for i, doc in enumerate(documents):
            for f, v in doc.items():
                if isinstance(v, dict):
                    del expanded[i][f]
                    for k in v:
                        expanded[i][f + "-" + k] = v[k]
        return expanded

    @staticmethod
    def remove_nan(documents: List[dict], replace_with: str = ""):
        for doc in documents:
            for f, v in doc.items():
                if isinstance(v, float) and math.isnan(v) or v == np.NaN:
                    doc[f] = replace_with
        return documents

    @staticmethod
    def build_range(document_count: int, chunk_size: int = 2000, start_idx: int = 0):
        rng = [
            (s, s + chunk_size)
            if s + chunk_size <= start_idx + document_count
            else (s, start_idx + document_count)
            for s in list(range(start_idx, start_idx + document_count, chunk_size))
        ]
        return rng

    def fetch_mongo_collection_data(self, start_idx: int = None, end_idx: int = None):
        if start_idx and end_idx:
            return list(self.mongo_collection.find()[start_idx:end_idx])
        return list(self.mongo_collection.find())

    def migrate(
        self,
        dataset_id: str,
        document_count: int,
        chunk_size: int = 2000,
        start_idx: int = 0,
        overwite: bool = False,
    ):
        """
        Migrate your MongoDB dataset ID.

        Parameters
        ------------
        dataset_id: str
            Name of your dataset
        document_count: int
            The number of documents in your collection
        chunk_size: int
            The number of chunks
        start_idx: int
            The start index in case it breaks
        overwrite: bool
            If True, then the dataset ID in Relevance AI will be overwritten

        """
        response = self.create_relevance_ai_dataset(dataset_id)
        if "already exists" in response["message"] and not overwite:
            self.logger.error(response["message"])
            return response["message"]

        total_ingest_cnt = 0
        for s_idx, e_idx in tqdm(
            MongoImporter.build_range(document_count, chunk_size, start_idx)
        ):
            df = pd.DataFrame(self.fetch_mongo_collection_data(s_idx, e_idx))
            documents = self.update_id(MongoImporter.parse_json(df.to_dict("records")))
            documents = MongoImporter.remove_nan(
                MongoImporter.flatten_inner_indxs(documents)
            )
            self.insert_documents(dataset_id, documents)
            total_ingest_cnt += len(documents)

        self.logger.info(
            f"Successfully ingested {total_ingest_cnt} entities to {dataset_id}."
        )
        return f"Successfully ingested {total_ingest_cnt} entities to {dataset_id}."



"""
Checks for available integrations in the Python module.
"""

import importlib


def is_faiss_available():
    return importlib.util.find_spec("faiss") is not None


def is_scipy_available():
    return importlib.util.find_spec("scipy") is not None


def is_sklearn_available():
    if importlib.util.find_spec("sklearn") is None:
        return False
    return is_scipy_available() and importlib.util.find_spec("sklearn.metrics")



from relevanceai.http_client import Dataset


def test_reduce_dimensions(test_df: Dataset):

    OUTPUT_VECTOR_FIELD = "sample_1_vector_"
    ALIAS = "pca"

    test_df.reduce_dimensions(vector_fields=[OUTPUT_VECTOR_FIELD], alias="pca")

    vector_field_name = ".".join(["_dr_", ALIAS, OUTPUT_VECTOR_FIELD])
    assert vector_field_name in test_df.schema, "Did not reduce dimensions properly"


def test_auto_reduce_dimensions(test_df: Dataset):

    OUTPUT_VECTOR_FIELD = "sample_1_vector_"
    ALIAS = "pca-3"

    test_df.auto_reduce_dimensions(vector_fields=[OUTPUT_VECTOR_FIELD], alias=ALIAS)

    vector_field_name = ".".join(["_dr_", ALIAS, OUTPUT_VECTOR_FIELD])
    assert vector_field_name in test_df.schema, "Did not reduce dimensions properly"



import random

import uuid

from typing import NamedTuple

from dataclasses import dataclass


@dataclass
class DataclassDocument:
    _id: str = uuid.uuid4().__str__()
    value1: float = random.random()
    value2: float = random.random()


def dataclass_document(_id: str) -> NamedTuple:
    return DataclassDocument(_id=_id)



import analytics
import asyncio

from typing import Callable
from functools import wraps

from relevanceai.config import CONFIG
from relevanceai.json_encoder import json_encoder


def enable_tracking():
    if CONFIG.is_field("mixpanel.enable_tracking"):
        return CONFIG.get_field("mixpanel.enable_tracking", CONFIG.config)


def track(func: Callable):
    @wraps(func)
    def wrapper(*args, **kwargs):
        try:
            if enable_tracking():

                async def send_analytics():
                    user_id = args[0].firebase_uid
                    event = f"pysdk-{func.__name__}"
                    kwargs.update(dict(zip(func.__code__.co_varnames, args)))
                    properties = {
                        "args": args[1:],
                        "kwargs": kwargs,
                    }
                    if user_id is not None:
                        analytics.track(
                            user_id=user_id,
                            event=json_encoder(event, force_string=True),
                            properties=json_encoder(properties, force_string=True),
                        )

                asyncio.ensure_future(send_analytics())
        except Exception as e:
            pass

        return func(*args, **kwargs)

    return wrapper


def identify(func: Callable):
    def wrapper(*args, **kwargs):
        try:
            if enable_tracking():
                user_id = args[0].firebase_uid
                region = args[0].region
                traits = {
                    "region": region,
                }
                if user_id is not None:
                    analytics.identify(user_id, traits)
        except Exception as e:
            pass

        return func(*args, **kwargs)

    return wrapper
